package dev.peerat.parser.java.visitor;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.visitor.operation.JavaOperationVisitor;
import dev.peerat.parser.java.visitor.value.JavaArrayAccessValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaArrayValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaBiValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaCastValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaInstanceValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaLambdaValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaMethodCallValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaModifierValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaStaticValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaTriValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaValueVisitor;
import dev.peerat.parser.java.visitor.value.JavaVariableAccessValueVisitor;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;

public abstract class JavaVisitor<T extends JavaElement> extends Visitor<JavaElement>{
	
	//TODO current system don't do litteraly ALL, juste DIRECT SUB ELEMENT
	
	public static <T extends JavaElement> JavaVisitor<T> base(Visitor<T> visitor){
		return new JavaVisitor<T>(null,true){
			
			@Override
			public boolean canVisit(Class<?> type){
//				System.out.println("base can visit ? "+type);
				return visitor.canVisit(type);
			}
			
			@Override
			public VisitorBag visitElement(T element){
				return visitor.visit(element);
			}
		};
	}
	
	private static JavaVisitor<JavaElement> javaElement(boolean propagate){
		return new JavaVisitor<JavaElement>(JavaElement.class, propagate){

			@Override
			public VisitorBag visitElement(JavaElement element){
				VisitorBag bag = new VisitorBag();
				bag.compute(element);
				return bag;
			}
			
		};
	}

	public static JavaVisitor<JavaElement> allJavaElement(){
		return javaElement(true);
	}
	
	public static JavaClassBaseVisitor allClassBase(){
		return new JavaClassBaseVisitor(true);
	}
	
	public static JavaClassVisitor allClass(){
		return new JavaClassVisitor(true);
	}
	
	public static JavaInterfaceVisitor allInterface(){
		return new JavaInterfaceVisitor(true);
	}
	
	public static JavaEnumerationVisitor allEnum(){
		return new JavaEnumerationVisitor(true);
	}
	
	public static JavaAnnotationClassVisitor allAnnotationClass(){
		return new JavaAnnotationClassVisitor(true);
	}
	
	public static JavaAnnotationVisitor allAnnotation(){
		return new JavaAnnotationVisitor(true);
	}
	
	public static JavaVariableVisitor allVariable(){
		return new JavaVariableVisitor(true);
	}
	
	public static JavaFunctionVisitor allFunction(){
		return new JavaFunctionVisitor(true);
	}
	
	public static JavaValueVisitor allValue(){
		return new JavaValueVisitor(true);
	}
	
	public static JavaArrayAccessValueVisitor allArrayAccessValue(){
		return new JavaArrayAccessValueVisitor(true);
	}
	
	public static JavaArrayValueVisitor allArrayValue(){
		return new JavaArrayValueVisitor(true);
	}
	
	public static JavaBiValueVisitor allBiValue(){
		return new JavaBiValueVisitor(true);
	}
	
	public static JavaCastValueVisitor allCastValue(){
		return new JavaCastValueVisitor(true);
	}
	
	public static JavaInstanceValueVisitor allInstanceValue(){
		return new JavaInstanceValueVisitor(true);
	}
	
	public static JavaLambdaValueVisitor allLambdaValue(){
		return new JavaLambdaValueVisitor(true);
	}
	
	public static JavaMethodCallValueVisitor allMethodCallValue(){
		return new JavaMethodCallValueVisitor(true);
	}
	
	public static JavaModifierValueVisitor allModifierValue(){
		return new JavaModifierValueVisitor(true);
	}
	
	public static JavaStaticValueVisitor allStaticValue(){
		return new JavaStaticValueVisitor(true);
	}
	
	public static JavaTriValueVisitor allTriValue(){
		return new JavaTriValueVisitor(true);
	}
	
	public static JavaVariableAccessValueVisitor allVariableAccessValue(){
		return new JavaVariableAccessValueVisitor(true);
	}
	
	public static JavaOperationVisitor allOperation(){
		return new JavaOperationVisitor(true);
	}
	
	public static JavaImportVisitor allImport(){
		return new JavaImportVisitor(true);
	}
	
	
	
	
	
	
	
	
	public static JavaVisitor<JavaElement> javaElement(){
		return javaElement(false);
	}
	
	public static JavaClassBaseVisitor classBase(){
		return new JavaClassBaseVisitor(false);
	}
	
	public static JavaClassVisitor clazz(){
		return new JavaClassVisitor(false);
	}
	
	public static JavaInterfaceVisitor interfaceClass(){
		return new JavaInterfaceVisitor(false);
	}
	
	public static JavaEnumerationVisitor enumClass(){
		return new JavaEnumerationVisitor(false);
	}
	
	public static JavaAnnotationClassVisitor annotationClass(){
		return new JavaAnnotationClassVisitor(false);
	}
	
	public static JavaAnnotationVisitor annotation(){
		return new JavaAnnotationVisitor(false);
	}
	
	public static JavaVariableVisitor variable(){
		return new JavaVariableVisitor(false);
	}
	
	public static JavaFunctionVisitor function(){
		return new JavaFunctionVisitor(false);
	}
	
	public static JavaValueVisitor value(){
		return new JavaValueVisitor(false);
	}
	
	public static JavaArrayAccessValueVisitor arrayAccessValue(){
		return new JavaArrayAccessValueVisitor(false);
	}
	
	public static JavaArrayValueVisitor arrayValue(){
		return new JavaArrayValueVisitor(false);
	}
	
	public static JavaBiValueVisitor biValue(){
		return new JavaBiValueVisitor(false);
	}
	
	public static JavaCastValueVisitor castValue(){
		return new JavaCastValueVisitor(false);
	}
	
	public static JavaInstanceValueVisitor instanceValue(){
		return new JavaInstanceValueVisitor(false);
	}
	
	public static JavaLambdaValueVisitor lambdaValue(){
		return new JavaLambdaValueVisitor(false);
	}
	
	public static JavaMethodCallValueVisitor methodCallValue(){
		return new JavaMethodCallValueVisitor(false);
	}
	
	public static JavaModifierValueVisitor modifierValue(){
		return new JavaModifierValueVisitor(false);
	}
	
	public static JavaStaticValueVisitor staticValue(){
		return new JavaStaticValueVisitor(false);
	}
	
	public static JavaTriValueVisitor triValue(){
		return new JavaTriValueVisitor(false);
	}
	
	public static JavaVariableAccessValueVisitor variableAccessValue(){
		return new JavaVariableAccessValueVisitor(false);
	}
	
	public static JavaOperationVisitor operation(){
		return new JavaOperationVisitor(false);
	}
	
	public static JavaImportVisitor imp(){
		return new JavaImportVisitor(false);
	}
	
	private Class<?> type;
	private boolean propagate;
	
	public JavaVisitor(Class<?> type, boolean propagate){
		this.type = type;
		this.propagate = propagate;
	}
	
	@Override
	public boolean canVisit(Class<?> type){
//		System.out.println("can "+type+" visit "+this.type+" ? from "+Thread.currentThread().getStackTrace()[2]);
		return this.type.isAssignableFrom(type);
	}
	
	@Override
	public boolean canPropagate(){
		return this.propagate;
	}
	
	@Override
	public VisitorBag visit(JavaElement element){
		return visitElement((T) element);
	}
	
	public abstract VisitorBag visitElement(T element);
}