package dev.peerat.parser.java.visitor.value;

import dev.peerat.parser.java.value.TriValue;
import dev.peerat.parser.visitor.Visitor;
import dev.peerat.parser.visitor.VisitorBag;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.java.value.Value;

public class JavaTriValueVisitor extends JavaVisitor<TriValue> {
	private Visitor<Value> checkPredicate;
	private Visitor<Value> successPredicate;
	private Visitor<Value> failPredicate;

	public JavaTriValueVisitor(boolean propagate) {
		super(TriValue.class, propagate);
	}

	public JavaTriValueVisitor check(Visitor<Value> visitor) {
		checkPredicate = visitor;
		return this;
	}

	public JavaTriValueVisitor success(Visitor<Value> visitor) {
		successPredicate = visitor;
		return this;
	}

	public JavaTriValueVisitor fail(Visitor<Value> visitor) {
		failPredicate = visitor;
		return this;
	}

	@Override
	public VisitorBag visitElement(TriValue element) {
		VisitorBag bag = new VisitorBag();

		if(checkPredicate != null){
			if(!checkPredicate.canVisit(element.getChecker().getClass())) return bag;
			VisitorBag visited = checkPredicate.visit(element.getChecker());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		if(successPredicate != null){
			if(!successPredicate.canVisit(element.success().getClass())) return bag;
			VisitorBag visited = successPredicate.visit(element.success());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		if(failPredicate != null){
			if(!failPredicate.canVisit(element.fail().getClass())) return bag;
			VisitorBag visited = failPredicate.visit(element.fail());
			if(!visited.isValidated()) return bag;
			bag.merge(visited);
		}
		
		bag.compute(element);
		return bag;
	}
}
